@@ -33,7 +33,8 @@ def rpad(value, width):
 	"""
 	strval = str(value)
 	if not isinstance(value, (int, float, long, complex)):
-		return '"%s"' % strval
+		# Need to escape single quote -> double quote
+		return '"%s"' % strval.replace('"', '""')
 	i = strval.find(".")
 	if i > -1:
 		if i < width - 1:
@@ -235,6 +236,7 @@ class CGATS(dict):
 	fileName = property(lambda self: self.filename,
 						lambda self, filename: setattr(self, "filename", filename))
 	key = None
+	_lvl = 0
 	_modified = False
 	mtime = None
 	parent = None
@@ -294,10 +296,38 @@ class CGATS(dict):
 				# strip control chars and leading/trailing whitespace
 				line = re.sub('[^\x09\x20-\x7E\x80-\xFF]', '', 
 								raw_line.strip())
-				comment_offset = line.find('#')
-				if comment_offset >= 0: # strip comment
-					line = line[:comment_offset].strip()
-				values = [value.strip('"') for value in line.split()]
+				if '#' in line or '"' in line:
+					# Deal with comments and quotes
+					quoted = False
+					values = []
+					token_start = 0
+					end = len(line) - 1
+					for i, char in enumerate(line):
+						if char == '"':
+							if quoted is False:
+								if not line[token_start:i]:
+									token_start = i
+								quoted = True
+							else:
+								quoted = False
+						if (quoted is False and char in '# \t') or i == end:
+							if i == end:
+								i += 1
+							value = line[token_start:i]
+							if value:
+								if value[0] == '"' == value[-1]:
+									# Unquote
+									value = value[1:-1]
+								# Need to unescape double quote -> single quote
+								values.append(value.replace('""', '"'))
+							if char == '#':
+								# Strip comment
+								line = line[:i].strip()
+								break
+							elif char in ' \t':
+								token_start = i + 1
+				else:
+					values = line.split()
 				if line[:6] == 'BEGIN_':
 					key = line[6:]
 					if key in context:
@@ -346,18 +376,25 @@ class CGATS(dict):
 					if values[0] == 'Date:':
 						context.datetime = line
 					else:
-						match = re.match(
-							'([^"]+?)(?:\s+("[^"]+"|[^\s]+))?(?:\s*#(.*))?$', 
-							line)
-						if match:
-							key, value, comment = match.groups()
+						if len(values) == 2 and not '"' in values[0]:
+							key, value = values
 							if value != None:
 								context = context.add_data({key: value.strip('"')})
 							else:
 								context = context.add_data({key: ''})
 				elif values and values[0] not in ('Comment:', 'Date:') and \
-				     len(line) >= 3 and not re.search("[^ 0-9A-Za-z]", line):
+				     len(line) >= 3 and not re.search("[^ 0-9A-Za-z/.]", line):
 					context = self.add_data(line)
+			if 0 in self and self[0].get("NORMALIZED_TO_Y_100") == "NO":
+				# Always normalize to Y = 100
+				reprstr = (self.filename or "<%s.%s instance at 0x%016x>" %
+											(self.__module__,
+											 self.__class__.__name__, id(self)))
+				if self[0].normalize_to_y_100():
+					safe_print("Normalized to Y = 100:", reprstr)
+				else:
+					safe_print("Warning: Could not normalize to Y = 100:",
+							   reprstr)
 			self.setmodified(False)
 
 	def __delattr__(self, name):
@@ -440,7 +477,7 @@ class CGATS(dict):
 		return desc
 
 	def __setattr__(self, name, value):
-		if name == '_keys':
+		if name in ('_keys', '_lvl'):
 			object.__setattr__(self, name, value)
 		elif name == 'modified':
 			self.setmodified(value)
@@ -466,6 +503,8 @@ class CGATS(dict):
 	
 	def __str__(self):
 		result = []
+		lvl = self.root._lvl
+		self.root._lvl += 1
 		data = None
 		if self.type == 'SAMPLE':
 			result.append(' '.join(rpad(self[item], 
@@ -504,6 +543,9 @@ class CGATS(dict):
 								key in self['KEYWORDS'].values():
 								if self.emit_keywords:
 									result.append('KEYWORD "%s"' % key)
+							if isinstance(value, basestring):
+								# Need to escape single quote -> double quote
+								value = value.replace('"', '""')
 							result.append('%s "%s"' % (key, value))
 				elif key not in ('DATA_FORMAT', 'KEYWORDS'):
 					if (value.type == 'SECTION' and result[-1:] and 
@@ -533,6 +575,11 @@ class CGATS(dict):
 										for item in 
 										data.parent['DATA_FORMAT'].values()]))
 			result.append('END_DATA')
+		if (self.parent and self.parent.type or
+			self.type) == 'ROOT' and result and result[-1] != '' and lvl == 0:
+			# Add empty line at end if not yet present
+			result.append('')
+		self.root._lvl -= 1
 		return '\n'.join(result)
 
 	def add_keyword(self, keyword, value=None):
@@ -902,7 +949,8 @@ class CGATS(dict):
 													  (item, type(value)))
 							else:
 								strval = str(abs(value))
-								if (item.startswith("RGB_") or
+								if (self.parent.type != "CAL" and
+									item.startswith("RGB_") or
 									item.startswith("CMYK_")):
 									# Assuming 0..100, 4 decimal digits is
 									# enough for roughly 19 bits integer
@@ -984,9 +1032,11 @@ class CGATS(dict):
 								key, value = var, data[var]
 						else:
 							key, value = len(self), var
-						if self.root.normalize_fields:
-							if isinstance(value, basestring):
-								value = value.upper()
+						if (self.root.normalize_fields and
+							(self.type in ('DATA_FORMAT', 'KEYWORDS') or
+							 var == 'KEYWORD') and
+							isinstance(value, basestring)):
+							value = value.upper()
 							if value == 'SAMPLEID':
 								value = 'SAMPLE_ID'
 							elif value == 'SAMPLENAME':
@@ -1646,12 +1696,62 @@ Transform {
 		self.setmodified()
 		return result
 
+	def convert_XYZ_to_Lab(self):
+		"""
+		Convert XYZ to D50 L*a*b* and add it as additional fields
+		
+		"""
+		color_rep = (self.queryv1("COLOR_REP") or "").split("_")
+
+		if color_rep[1] == "LAB":
+			# Nothing to do
+			return
+
+		if (len(color_rep) != 2 or color_rep[0] not in ("RGB", "CMYK") or
+			color_rep[1] != "XYZ"):
+			raise NotImplementedError("Got unsupported color representation %s"
+									  % "_".join(color_rep))
+
+		data = self.queryv1("DATA")
+		if not data:
+			raise CGATSError("No data")
+
+		if color_rep[0] == "RGB":
+			white = data.queryv1({"RGB_R": 100, "RGB_G": 100, "RGB_B": 100})
+		elif color_rep[0] == "CMYK":
+			white = data.queryv1({"CMYK_C": 0, "CMYK_M": 0, "CMYK_Y": 0,
+								  "CMYK_K": 0})
+		if not white:
+			raise CGATSError("Missing white patch")
+
+		device_labels = []
+		for channel in color_rep[0]:
+			device_labels.append(color_rep[0] + "_" + channel)
+
+		# Always XYZ
+		cie_labels = []
+		for channel in color_rep[1]:
+			cie_labels.append(color_rep[1] + "_" + channel)
+
+		# Add entries to DATA_FORMAT
+		Lab_data_format = ("LAB_L", "LAB_A", "LAB_B")
+		for label in Lab_data_format:
+			if not label in data.parent.DATA_FORMAT.values():
+				data.parent.DATA_FORMAT.add_data((label, ))
+
+		# Add L*a*b* to each sample
+		for key, sample in data.iteritems():
+			cie_values = [sample[label] for label in cie_labels]
+			Lab = colormath.XYZ2Lab(*cie_values)
+			for i, label in enumerate(Lab_data_format):
+				sample[label] = Lab[i]
+
 	def fix_zero_measurements(self, warn_only=False, logfile=safe_print):
 		"""
-		Fix (or warn about) zero measurements
+		Fix (or warn about) <= zero measurements
 		
 		If XYZ/Lab = 0, the sample gets removed. If only one component of
-		XYZ/Lab is 0, it gets fudged so that the component is nonzero
+		XYZ/Lab is <= 0, it gets fudged so that the component is nonzero
 		(because otherwise, Argyll's colprof will remove it, which can have bad
 		effects if it's an 'essential' sample)
 		
@@ -1663,57 +1763,57 @@ Transform {
 			cie_labels = []
 			for channel in color_rep[1]:
 				cie_labels.append(color_rep[1] + "_" + channel)
+				if color_rep[1] == "LAB":
+					# Only check L* for zero values
+					break
 			device_labels = []
 			for channel in color_rep[0]:
 				device_labels.append(color_rep[0] + "_" + channel)
-			errors = []
 			remove = []
 			for key, sample in data.iteritems():
 				cie_values = [sample[label] for label in cie_labels]
 				# Check if zero
 				if filter(lambda v: v, cie_values):
-					# Not zero. Check if one component zero
-					if 0.0 in cie_values:
+					# Not all zero. Check if some component(s) equal or below zero
+					if min(cie_values) <= 0:
 						for label in cie_labels:
-							if not sample[label]:
+							if sample[label] <= 0:
 								if warn_only:
 									if logfile:
-										logfile.write("Warning: Sample ID %i (RGB %s) has %s = 0!\n" %
+										logfile.write("Warning: Sample ID %i (%s %s) has %s <= 0!\n" %
 													  (sample.SAMPLE_ID,
+													   color_rep[0],
 													   " ".join(str(sample.queryv1(device_labels)).split()),
 													   label))
 								else:
 									# Fudge to be nonzero
 									sample[label] = 0.000001
 									if logfile:
-										logfile.write("Fudged sample ID %i (RGB %s) %s to be non-zero\n" %
+										logfile.write("Fudged sample ID %i (%s %s) %s to be non-zero\n" %
 													  (sample.SAMPLE_ID,
+													   color_rep[0],
 													   " ".join(str(sample.queryv1(device_labels)).split()),
 													   label))
 					continue
-				# Zero
+				# All zero
 				device_values = [sample[label] for label in device_labels]
-				device_sum = 0
-				for value in device_values:
-					if value > 0:
-						# Error on device values > 0
-						errors.append(sample)
-					device_sum += value
-				if not device_sum:
+				if not max(device_values):
 					# Skip device black
 					continue
 				if warn_only:
 					if logfile:
-						logfile.write("Warning: Sample ID %i (RGB %s) has %s = 0!\n" %
+						logfile.write("Warning: Sample ID %i (%s %s) has %s = 0!\n" %
 									  (sample.SAMPLE_ID,
+									   color_rep[0],
 									   " ".join(str(sample.queryv1(device_labels)).split()),
 									   color_rep[1]))
 				else:
 					# Queue sample for removal
 					remove.insert(0, sample)
 					if logfile:
-						logfile.write("Removed sample ID %i (RGB %s) with %s = 0\n" %
+						logfile.write("Removed sample ID %i (%s %s) with %s = 0\n" %
 									  (sample.SAMPLE_ID,
+									   color_rep[0],
 									   " ".join(str(sample.queryv1(device_labels)).split()),
 									   color_rep[1]))
 			for sample in remove:
@@ -1737,6 +1837,45 @@ Transform {
 							fixed += 1
 							break
 		return fixed
+
+	def normalize_to_y_100(self):
+		""" Scale XYZ values so that RGB 100 = Y 100 """
+		if "DATA" in self:
+			white_cie = self.get_white_cie()
+			if white_cie and "XYZ_Y" in white_cie:
+				white_Y = white_cie["XYZ_Y"]
+				if white_Y != 100:
+					self.add_keyword("LUMINANCE_XYZ_CDM2",
+									 "%.4f %.4f %.4f" % (white_cie["XYZ_X"],
+														 white_cie["XYZ_Y"],
+														 white_cie["XYZ_Z"]))
+					for sample in self.DATA.itervalues():
+						for label in "XYZ":
+							v = sample["XYZ_" + label]
+							sample["XYZ_" + label] = v / white_Y * 100
+				self.add_keyword("NORMALIZED_TO_Y_100", "YES")
+				return True
+		return False
+	
+	def quantize_device_values(self, bits=8, quantizer=round):
+		""" Quantize device values to n bits """
+		q = 2 ** bits - 1.0
+		for data in self.queryv("DATA").itervalues():
+			if data.parent.type == "CAL":
+				maxv = 1.0
+				digits = 8
+			else:
+				maxv = 100.0
+				# Assuming 0..100, 4 decimal digits is
+				# enough for roughly 19 bits integer
+				# device values
+				digits = 4
+			color_rep = (data.parent.queryv1("COLOR_REP") or "").split("_")[0]
+			for labels in get_device_value_labels(color_rep):
+				for item in data.queryi(labels).itervalues():
+					for label in labels:
+						item[label] = round(quantizer(item[label] / maxv * q) /
+											q * maxv, digits)
 	
 	def scale_device_values(self, factor=100.0 / 255, color_rep=None):
 		""" Scales device values by multiplying with factor. """
