@@ -262,6 +262,8 @@ def getbitmap(name, display_missing_icon=True, scale=True, use_mask=False):
 			w = int(round(w * scale))
 			h = int(round(h * scale))
 		if parts[-1] == "empty":
+			if wx.VERSION < (3, ):
+				use_mask = True
 			if use_mask and sys.platform == "win32":
 				bmp = wx.EmptyBitmap(w, h)
 				bmp.SetMaskColour(wx.Colour(0, 0, 0))
@@ -323,7 +325,7 @@ def getbitmap(name, display_missing_icon=True, scale=True, use_mask=False):
 							# Try original size
 							parts[-1] = oname
 				if (sys.platform not in ("darwin", "win32") and
-					parts[-1].startswith(appname)):
+					parts[-1].startswith(appname.lower())):
 					# Search /usr/share/icons on Linux first
 					path = get_data_path(os.path.join(parts[-2], "apps", parts[-1]) + ".png")
 				if not path:
@@ -414,6 +416,8 @@ def getbitmap(name, display_missing_icon=True, scale=True, use_mask=False):
 					if not bmp.IsOk():
 						path = None
 			if not path:
+				from log import safe_print
+				safe_print("Warning: Missing bitmap '%s'" % name)
 				img = wx.EmptyImage(w, h)
 				img.SetMaskColour(0, 0, 0)
 				img.InitAlpha()
@@ -702,20 +706,22 @@ cfg = ConfigParser.RawConfigParser()
 cfg.optionxform = str
 
 valid_ranges = {
-	"3dlut.hdr_peak_luminance": [120.0, 10000.0],
+	"3dlut.hdr_peak_luminance": [100.0, 10000.0],
 	"3dlut.hdr_minmll": [0.0, 0.1],
-	"3dlut.hdr_maxmll": [120.0, 10000.0],
+	"3dlut.hdr_maxmll": [100.0, 10000.0],
 	"3dlut.trc_gamma": [0.000001, 10],
+	"3dlut.hdr_sat": [0.0, 1.0],
+	"3dlut.hdr_hue": [0.0, 1.0],
 	"3dlut.trc_output_offset": [0.0, 1.0],
 	"app.port": [1, 65535],
 	"gamma": [0.000001, 10],
 	"trc": [0.000001, 10],
 	"calibration.ambient_viewcond_adjust.lux": [0, sys.maxint],
-	"calibration.black_luminance": [0.000001, 100000],
+	"calibration.black_luminance": [0.000001, 10],
 	"calibration.black_output_offset": [0, 1],
 	"calibration.black_point_correction": [0, 1],
 	"calibration.black_point_rate": [0.05, 20],
-	"calibration.luminance": [0.000001, 100000],
+	"calibration.luminance": [20, 100000],
 	"iccgamut.surface_detail": [1.0, 50.0],
 	"measurement_report.trc_gamma": [0.01, 10],
 	"measurement_report.trc_output_offset": [0.0, 1.0],
@@ -723,6 +729,10 @@ valid_ranges = {
 	"measure.min_display_update_delay_ms": [20, 60000],
 	"multiprocessing.max_cpus": [0, 65],
 	"patterngenerator.apl": [0.0, 1.0],
+	"patterngenerator.ffp_insertion.duration": [1, 60],
+	"patterngenerator.ffp_insertion.interval": [1, 3600],
+	"patterngenerator.ffp_insertion.level": [0.0, 1.0],
+	"patterngenerator.quantize_bits": [0, 32],
 	"patterngenerator.resolve.port": [1, 65535],
 	"profile_loader.quantize_bits": [8, 16],
 	"synthprofile.trc_gamma": [0.01, 10],
@@ -808,7 +818,7 @@ valid_values = {
 	"whitepoint.colortemp.locus": ["t", "T"]
 }
 
-content_rgb_space = colormath.get_rgb_space("DCI P3")
+content_rgb_space = colormath.get_rgb_space("DCI P3 D65")
 crx, cry = content_rgb_space[2:][0][:2]
 cgx, cgy = content_rgb_space[2:][1][:2]
 cbx, cby = content_rgb_space[2:][2][:2]
@@ -842,8 +852,11 @@ defaults = {
 	"3dlut.hdr_display": 0,
 	"3dlut.hdr_minmll": 0.0,
 	"3dlut.hdr_maxmll": 10000.0,
-	"3dlut.hdr_peak_luminance": 400.0,
+	"3dlut.hdr_maxmll_alt_clip": 1,
+	"3dlut.hdr_peak_luminance": 480.0,
 	"3dlut.hdr_ambient_luminance": 5.0,
+	"3dlut.hdr_sat": 0.5,
+	"3dlut.hdr_hue": 0.5,
 	"3dlut.image.layout": "h",
 	"3dlut.image.order": "rgb",
 	"3dlut.input.profile": "",
@@ -984,10 +997,15 @@ defaults = {
 	"observer.backup": "1931_2",
 	"patterngenerator.apl": .22,
 	"patterngenerator.detect_video_levels": 1,
+	"patterngenerator.ffp_insertion": 0,
+	"patterngenerator.ffp_insertion.duration": 5,
+	"patterngenerator.ffp_insertion.interval": 5,
+	"patterngenerator.ffp_insertion.level": 0.15,
 	"patterngenerator.prisma.argyll": 0,
 	"patterngenerator.prisma.host": "",
 	"patterngenerator.prisma.preset": "Custom-1",
 	"patterngenerator.prisma.port": 80,
+	"patterngenerator.quantize_bits": 0,
 	"patterngenerator.resolve": "CM",
 	"patterngenerator.resolve.port": 20002,
 	"patterngenerator.use_video_levels": 0,
@@ -1628,7 +1646,7 @@ def initcfg(module=None):
 		cfgnames.append(cfgbasename)
 	else:
 		cfgnames.extend("%s-%s" % (appbasename, othermod) for othermod in
-						("synthprofile", "testchart-editor"))
+						("testchart-editor", ))
 	cfgroots = [confighome]
 	if module == "apply-profiles":
 		cfgroots.append(config_sys)
@@ -1653,7 +1671,7 @@ def initcfg(module=None):
 		safe_print("Warning - could not parse configuration files:\n%s" %
 				   "\n".join(cfgfiles))
 		# Fix Python 2.7 ConfigParser option values being lists instead of
-		# strings in case of a ParsingError. http://bugs.python.org/issue2414
+		# strings in case of a ParsingError. http://bugs.python.org/issue24142
 		all_sections = [ConfigParser.DEFAULTSECT]
 		all_sections.extend(cfg.sections())
 		for section in all_sections:
@@ -1789,9 +1807,19 @@ def writecfg(which="user", worker=None, module=None, options=()):
 _init_testcharts()
 runtype = runtimeconfig(pyfile)
 
-if isapp and not os.getenv("SSL_CERT_FILE"):
-	# Use our bundled CA file
-	# https://github.com/certifi/python-certifi/blob/master/certifi/cacert.pem
-	cafile = get_data_path("cacert.pem")
+if sys.platform in ("darwin", "win32") and not os.getenv("SSL_CERT_FILE"):
+	try:
+		import certifi
+	except ImportError:
+		cafile = None
+	else:
+		cafile = certifi.where()
+		if cafile and not os.path.isfile(cafile):
+			cafile = None
+	if not cafile:
+		# Use our bundled CA file
+		cafile = get_data_path("cacert.pem")
+		if cafile:
+			cafile = cafile.encode(fs_enc, "replace")
 	if cafile:
-		os.environ["SSL_CERT_FILE"] = cafile.encode(fs_enc, "replace")
+		os.environ["SSL_CERT_FILE"] = cafile
