@@ -15,6 +15,7 @@ import ICCProfile as ICCP
 import config
 import localization as lang
 import worker
+from util_list import natsort_key_factory
 from wxTestchartEditor import TestchartEditor
 from wxwindows import BaseApp, BaseFrame, FileDrop, InfoDialog, wx
 from wxfixes import TempXmlResource
@@ -90,8 +91,9 @@ class ReportFrame(BaseFrame):
 						if not (path.lower().endswith(".ti2") and
 								basepath + ".cie" in paths):
 							history.append(path)
+				natsort_key = natsort_key_factory()
 				history = sorted(history, key=lambda path:
-											  os.path.basename(path).lower())
+											  natsort_key(os.path.basename(path)))
 				ctrl.SetHistory(history)
 			ctrl.SetMaxFontSize(11)
 			# Drop targets
@@ -197,7 +199,7 @@ class ReportFrame(BaseFrame):
 		if v != getcfg("measurement_report.trc_output_offset"):
 			setcfg("measurement_report.trc_output_offset", v)
 			self.mr_update_trc_control()
-			self.mr_show_trc_controls()
+			#self.mr_show_trc_controls()
 
 	def mr_trc_gamma_ctrl_handler(self, event):
 		try:
@@ -231,6 +233,12 @@ class ReportFrame(BaseFrame):
 			setcfg("measurement_report.trc_gamma_type", "b")
 			setcfg("measurement_report.trc_output_offset", 1.0)
 			self.mr_update_trc_controls()
+		else:
+			# Custom
+			# Have to use CallAfter, otherwise only part of the text will
+			# be selected (wxPython bug?)
+			wx.CallAfter(self.mr_trc_gamma_ctrl.SetFocus)
+			wx.CallLater(1, self.mr_trc_gamma_ctrl.SelectAll)
 		self.mr_show_trc_controls()
 		self.Thaw()
 
@@ -311,10 +319,6 @@ class ReportFrame(BaseFrame):
 					self.chart_patches_amount.GetContainingSizer().Layout()
 					self.chart_patches_amount.Thaw()
 					self.chart_white = cgats.get_white_cie()
-					if event:
-						v = int(not self.chart_white or not "RGB" in values)
-						setcfg("measurement_report.whitepoint.simulate", v)
-						setcfg("measurement_report.whitepoint.simulate.relative", v)
 			if not values:
 				if chart:
 					show_result_dialog(lang.getstr("error.testchart.missing_fields",
@@ -338,6 +342,17 @@ class ReportFrame(BaseFrame):
 		self.fields_ctrl.Enable(self.fields_ctrl.GetCount() > 1)
 		self.fields_ctrl_handler(event)
 
+	def set_simulate_whitepoint(self, set_whitepoint_simulate_relative=False):
+		sim_profile = self.get_simulation_profile()
+		is_prtr_profile = sim_profile and sim_profile.profileClass == "prtr"
+		if set_whitepoint_simulate_relative:
+			setcfg("measurement_report.whitepoint.simulate",
+					int(not getattr(self, "chart_white", None) or
+						not "RGB" in self.fields_ctrl.Items or
+						is_prtr_profile))
+		setcfg("measurement_report.whitepoint.simulate.relative",
+			   int("LAB" in self.fields_ctrl.Items or is_prtr_profile))
+
 	def chart_drop_handler(self, path):
 		if not self.worker.is_working():
 			self.chart_ctrl.SetPath(path)
@@ -360,7 +375,7 @@ class ReportFrame(BaseFrame):
 		setcfg("measurement_report.chart.fields",
 			   self.fields_ctrl.GetStringSelection())
 		if event:
-			self.mr_update_main_controls()
+			self.mr_update_main_controls(event)
 	
 	def output_profile_ctrl_handler(self, event):
 		self.set_profile("output")
@@ -468,7 +483,17 @@ class ReportFrame(BaseFrame):
 													   % odata, self)
 									self.set_profile_ctrl_path(which)
 									return
-								self.XYZbpout = odata[0]
+								if odata[0][1]:
+									# Got above zero blackpoint from lookup
+									self.XYZbpout = odata[0]
+								else:
+									# Got zero blackpoint from lookup.
+									# Try chardata instead.
+									XYZbp = profile.get_chardata_bkpt()
+									if XYZbp:
+										self.XYZbpout = XYZbp
+									else:
+										self.XYZbpout = [0, 0, 0]
 					else:
 						# Profile selection has not changed
 						# Restore cached XYZbp values
@@ -590,7 +615,11 @@ class ReportFrame(BaseFrame):
 		self.set_profile("devlink", silent=True)
 		self.output_profile_ctrl.SetPath(getcfg("measurement_report.output_profile"))
 		self.set_profile("output", silent=True)
-		self.mr_set_testchart(getcfg("measurement_report.chart"))
+		chart = getcfg("measurement_report.chart")
+		if not chart or not os.path.isfile(chart):
+			chart = config.defaults["measurement_report.chart"]
+			setcfg("measurement_report.chart", chart)
+		self.mr_set_testchart(chart)
 		self.use_simulation_profile_ctrl_handler(None, update_trc=False)
 		self.panel.Thaw()
 
@@ -618,7 +647,7 @@ class ReportFrame(BaseFrame):
 		self.chart_ctrl.SetPath(path)
 		self.chart_ctrl_handler(None)
 	
-	def mr_update_main_controls(self):
+	def mr_update_main_controls(self, event=None):
 		##print "MR update main ctrls",
 		self.panel.Freeze()
 		chart_has_white = bool(getattr(self, "chart_white", None))
@@ -651,7 +680,9 @@ class ReportFrame(BaseFrame):
 				   isinstance(self.simulation_profile.tags.get("gXYZ"),
 							  ICCP.XYZType) and
 				   isinstance(self.simulation_profile.tags.get("bXYZ"),
-							  ICCP.XYZType))
+							  ICCP.XYZType) and
+				   not isinstance(self.simulation_profile.tags.get("A2B0"),
+							  ICCP.LUT16Type))
 		##print enable5, self.XYZbpin, self.XYZbpout
 		self.mr_trc_label.Show(enable1 and enable5)
 		self.apply_none_ctrl.Show(enable1 and enable5)
@@ -674,6 +705,8 @@ class ReportFrame(BaseFrame):
 				self.XYZbpout > self.XYZbpin)
 		self.input_value_clipping_bmp.Show(show)
 		self.input_value_clipping_label.Show(show)
+		if event:
+			self.set_simulate_whitepoint(True)
 		self.simulate_whitepoint_cb.Enable((enable1 and not enable2) or
 										   (color in ("LAB", "XYZ") and
 											chart_has_white))
@@ -822,27 +855,27 @@ class ReportFrame(BaseFrame):
 		setcfg("measurement_report.use_devlink_profile",
 			   int(self.devlink_profile_cb.GetValue()))
 		self.mr_update_main_controls()
+
+	def get_simulation_profile(self):
+		""" Return simulation profile if enabled """
+		use_sim_profile = getcfg("measurement_report.use_simulation_profile")
+		return use_sim_profile and getattr(self, "simulation_profile", None)
 	
 	def use_simulation_profile_ctrl_handler(self, event, update_trc=True):
 		if event:
-			use_sim_profile = self.simulation_profile_cb.GetValue()
 			setcfg("measurement_report.use_simulation_profile",
-				   int(use_sim_profile))
-		else:
-			use_sim_profile = getcfg("measurement_report.use_simulation_profile")
-		sim_profile = getattr(self, "simulation_profile", None)
+				   int(self.simulation_profile_cb.GetValue()))
+		sim_profile = self.get_simulation_profile()
 		enable = False
 		if sim_profile:
-			v = int(sim_profile.profileClass == "prtr")
-			setcfg("measurement_report.whitepoint.simulate", v)
-			setcfg("measurement_report.whitepoint.simulate.relative", v)
+			self.set_simulate_whitepoint()
 			if ("rTRC" in sim_profile.tags and "gTRC" in sim_profile.tags and
 				"bTRC" in sim_profile.tags and sim_profile.tags.rTRC ==
 				sim_profile.tags.gTRC == sim_profile.tags.bTRC and
 				isinstance(sim_profile.tags.rTRC, ICCP.CurveType)):
-				tf = sim_profile.tags.rTRC.get_transfer_function()
+				tf = sim_profile.tags.rTRC.get_transfer_function(outoffset=1.0)
 				if update_trc or self.XYZbpin == self.XYZbpout:
-					# Use only BT.1886 black output offset
+					# Use only BT.1886 black output offset or not
 					setcfg("measurement_report.apply_black_offset",
 						   int(tf[0][1] not in (-240, -709) and
 							   (not tf[0][0].startswith("Gamma") or tf[1] < .95) and
@@ -867,7 +900,11 @@ class ReportFrame(BaseFrame):
 				self.mr_update_trc_controls()
 				enable = (tf[0][1] not in (-240, -709) and
 						  self.XYZbpin != self.XYZbpout)
-		self.apply_black_offset_ctrl.Enable(use_sim_profile and enable)
+			elif update_trc:
+				enable = self.XYZbpin != self.XYZbpout
+				setcfg("measurement_report.apply_black_offset", int(enable))
+				setcfg("measurement_report.apply_trc", 0)
+		self.apply_black_offset_ctrl.Enable(bool(sim_profile) and enable)
 		self.mr_update_main_controls()
 
 	def use_simulation_profile_as_output_handler(self, event):
