@@ -20,7 +20,7 @@ import colormath
 import config
 import localization as lang
 import worker
-from wxwindows import BaseApp, BaseFrame, FileDrop, InfoDialog, wx
+from wxwindows import BaseApp, BaseFrame, ConfirmDialog, FileDrop, InfoDialog, wx
 from wxfixes import TempXmlResource
 from wxLUT3DFrame import LUT3DFrame
 import floatspin
@@ -70,6 +70,8 @@ class SynthICCFrame(BaseFrame):
 		presets = [""] + sorted(colormath.rgb_spaces.keys())
 		self.preset_ctrl.SetItems(presets)
 
+		self.set_default_cat()
+
 		self.worker = worker.Worker(self)
 
 		# Drop targets
@@ -92,6 +94,8 @@ class SynthICCFrame(BaseFrame):
 						  getattr(self, "%s_%s" % (color, component)))
 		self.black_point_cb.Bind(wx.EVT_CHECKBOX,
 								 self.black_point_enable_handler)
+		self.chromatic_adaptation_btn.Bind(wx.EVT_BUTTON,
+										   self.chromatic_adaptation_btn_handler)
 		self.trc_ctrl.Bind(wx.EVT_CHOICE, self.trc_ctrl_handler)
 		self.trc_gamma_ctrl.Bind(wx.EVT_COMBOBOX, self.trc_gamma_ctrl_handler)
 		self.trc_gamma_ctrl.Bind(wx.EVT_KILL_FOCUS, self.trc_gamma_ctrl_handler)
@@ -112,7 +116,6 @@ class SynthICCFrame(BaseFrame):
 		self.save_as_btn.Bind(wx.EVT_BUTTON, self.save_as_btn_handler)
 
 		self.save_as_btn.SetDefault()
-		self.save_as_btn.Disable()
 		
 		self.setup_language()
 		
@@ -169,8 +172,8 @@ class SynthICCFrame(BaseFrame):
 				wx.Bell()
 			v = white_Y * .9
 		if event:
-			min_Y = (1 / 65535.0) * 100
-			increment = (1 / 65535.0) * white_Y
+			min_Y = 0.000001 #(1 / 65535.0) * 100
+			increment = 0.000001 #(1 / 65535.0) * white_Y
 			if increment < min_Y:
 				increment = min_Y * (white_Y / 100.0)
 			min_inc = 1.0 / (10.0 ** self.black_luminance_ctrl.GetDigits())
@@ -240,8 +243,7 @@ class SynthICCFrame(BaseFrame):
 					for i in xrange(3):
 						getattr(self, "black_%s" % "XYZ"[i]).SetValue(0)
 					break
-		for i, v in enumerate(colormath.XYZ2xyY(*XYZ)[:2]):
-			getattr(self, "black_%s" % "xy"[i]).SetValue(v)
+		self.parse_XYZ("black")
 
 	def black_xy_ctrl_handler(self, event):
 		# Black Y scaled to 0..1 range
@@ -260,15 +262,83 @@ class SynthICCFrame(BaseFrame):
 	
 	def blue_xy_ctrl_handler(self, event):
 		self.parse_xy("blue")
+
+	def chromatic_adaptation_btn_handler(self, event):
+		scale = getcfg("app.dpi") / config.get_default_dpi()
+		if scale < 1:
+			scale = 1
+		dlg = ConfirmDialog(self, title=lang.getstr("chromatic_adaptation"),
+							msg=lang.getstr("whitepoint.xy"),
+							ok=lang.getstr("apply"),
+							cancel=lang.getstr("cancel"))
+		sizer = wx.BoxSizer(wx.HORIZONTAL)
+		dlg.sizer3.Add(sizer, 0, flag=wx.TOP | wx.ALIGN_LEFT,
+					   border=8)
+		x_ctrl = floatspin.FloatSpin(dlg, -1, size=(75 * scale, -1),
+									 min_val=0.0001, max_val=1,
+									 value=self.white_x.GetValue())
+		sizer.Add(x_ctrl, 0, flag=wx.RIGHT | wx.ALIGN_CENTER_VERTICAL,
+				  border=4)
+		sizer.Add(wx.StaticText(dlg, -1, u"x"), 0,
+								flag=wx.RIGHT | wx.ALIGN_CENTER_VERTICAL,
+								border=12)
+		y_ctrl = floatspin.FloatSpin(dlg, -1, size=(75 * scale, -1),
+									 min_val=0.0001, max_val=1,
+									 value=self.white_y.GetValue())
+		sizer.Add(y_ctrl, 0, flag=wx.RIGHT | wx.ALIGN_CENTER_VERTICAL,
+				  border=4)
+		sizer.Add(wx.StaticText(dlg, -1, u"y"), 0,
+								flag=wx.RIGHT | wx.ALIGN_CENTER_VERTICAL,
+								border=12)
+		dlg.sizer3.Add(wx.StaticText(dlg, -1,
+									 lang.getstr("chromatic_adaptation_transform")),
+					   0, flag=wx.TOP | wx.ALIGN_LEFT, border=12)
+		if getcfg("show_advanced_options"):
+			# Only offer those that seem suitable in an ICC workflow
+			# (i.e. blue primary not too far from default Bradford to
+			# prevent 'blue turns purple' problems)
+			cat_choices = ['Bradford', 'CAT02BS']
+		else:
+			cat_choices = ["Bradford"]
+		cat_choices_ab = OrderedDict(get_mapping(((k, k) for k in
+												  colormath.cat_matrices),
+												 cat_choices))
+		cat_choices_ba = OrderedDict((v, k) for k, v in cat_choices_ab.iteritems())
+		cat_ctrl = wx.Choice(dlg, -1, choices=cat_choices_ab.values())
+		cat_ctrl.SetStringSelection(cat_choices_ab[self.cat])
+		dlg.sizer3.Add(cat_ctrl, 0, flag=wx.TOP | wx.ALIGN_LEFT, border=8)
+		dlg.sizer0.SetSizeHints(dlg)
+		dlg.sizer0.Layout()
+		result = dlg.ShowModal()
+		x = x_ctrl.GetValue()
+		y = y_ctrl.GetValue()
+		cat = cat_choices_ba[cat_ctrl.GetStringSelection()]
+		dlg.Destroy()
+		if result != wx.ID_OK:
+			return
+		wp_src = [getattr(self, "white_" + component).GetValue()
+				  for component in "XYZ"]
+		wp_tgt = colormath.xyY2XYZ(x, y)
+		self.cat = cat
+		for color in ("red", "green", "blue", "white", "black"):
+			ctrls = [getattr(self, "%s_%s" % (color, component))
+					 for component in "XYZ"]
+			X, Y, Z = (ctrl.GetValue() for ctrl in ctrls)
+			XYZa = colormath.adapt(X, Y, Z, wp_src, wp_tgt, cat)
+			for i, ctrl in enumerate(ctrls):
+				ctrl.SetValue(XYZa[i])
+			self.parse_XYZ(color, False)
 	
 	def colorspace_ctrl_handler(self, event):
+		self.Freeze()
 		show = bool(self.colorspace_rgb_ctrl.Value)
 		for color in ("red", "green", "blue"):
 			getattr(self, "label_%s" % color).Show(show)
 			for component in "XYZxy":
 				getattr(self, "%s_%s" % (color, component)).Show(show)
-		self.enable_save_as_btn()
+		self.enable_btns()
 		self.update_layout()
+		self.Thaw()
 	
 	def drop_handler(self, path):
 		try:
@@ -277,7 +347,8 @@ class SynthICCFrame(BaseFrame):
 			show_result_dialog(Error(lang.getstr("profile.invalid") + "\n" +
 									 path), self)
 		else:
-			if profile.version >= 4:
+			if (profile.version >= 4 and
+				not profile.convert_iccv4_tags_to_iccv2()):
 				show_result_dialog(Error(lang.getstr("profile.iccv4.unsupported")),
 								   self)
 				return
@@ -310,6 +381,14 @@ class SynthICCFrame(BaseFrame):
 				setcfg("synthprofile.luminance", luminance)
 				self.luminance_ctrl.SetValue(luminance)
 				for i, color in enumerate(("white", "black")):
+					if (color == "black" and not colors[i][1] and
+						isinstance(profile.tags.get("targ"), ICCP.Text)):
+						# The profile may not reflect the actual black point.
+						# Get it from the embedded TI3 instead if zero from lookup.
+						XYZbp = profile.get_chardata_bkpt(True)
+						if XYZbp:
+							# Use wtpt chromaticity
+							colors[i] = [v * XYZbp[1] for v in colors[0]]
 					for j, component in enumerate("XYZ"):
 						getattr(self, "%s_%s" %
 								(color, component)).SetValue(colors[i][j] /
@@ -337,8 +416,10 @@ class SynthICCFrame(BaseFrame):
 				self.trc_gamma_type_ctrl.SetSelection(self.trc_gamma_types_ba["g"])
 				self.panel.Thaw()
 	
-	def enable_save_as_btn(self):
-		self.save_as_btn.Enable(bool(self.get_XYZ()))
+	def enable_btns(self):
+		enable = bool(self.get_XYZ())
+		self.save_as_btn.Enable(enable)
+		self.chromatic_adaptation_btn.Enable(enable)
 	
 	def get_XYZ(self):
 		""" Get XYZ in 0..1 range """
@@ -373,8 +454,8 @@ class SynthICCFrame(BaseFrame):
 		self.lut3d_set_option("3dlut.hdr_peak_luminance", v)
 		self.black_luminance_ctrl_handler(event)
 	
-	def parse_XYZ(self, name, set_blackpoint=False):
-		if not set_blackpoint:
+	def parse_XYZ(self, name, set_blackpoint=None):
+		if set_blackpoint is None:
 			set_blackpoint = not self.black_point_cb.Value
 		if not self._updating_ctrls:
 			self.preset_ctrl.SetSelection(0)
@@ -388,12 +469,18 @@ class SynthICCFrame(BaseFrame):
 			if name == "white" and set_blackpoint:
 				getattr(self, "black_%s" % (component)).SetValue(v * black_Y)
 		if "X" in XYZ and "Y" in XYZ and "Z" in XYZ:
-			xyY = colormath.XYZ2xyY(XYZ["X"], XYZ["Y"], XYZ["Z"])
+			if XYZ["X"] + XYZ["Y"] + XYZ["Z"] == 0:
+				# Set black chromaticity to white chromaticity if XYZ is 0
+				xyY = []
+				for i, component in enumerate("xy"):
+					xyY.append(getattr(self, "white_%s" % component).GetValue())
+			else:
+				xyY = colormath.XYZ2xyY(XYZ["X"], XYZ["Y"], XYZ["Z"])
 			for i, component in enumerate("xy"):
 				getattr(self, "%s_%s" % (name, component)).SetValue(xyY[i])
 				if name == "white" and set_blackpoint:
 					getattr(self, "black_%s" % (component)).SetValue(xyY[i])
-		self.enable_save_as_btn()
+		self.enable_btns()
 	
 	def parse_xy(self, name=None, set_blackpoint=False):
 		if not set_blackpoint:
@@ -446,11 +533,12 @@ class SynthICCFrame(BaseFrame):
 			for i, component in enumerate("XYZ"):
 				getattr(self, "%s_%s" %
 						(color, component)).SetValue(XYZ[color[0]][i] * 100)
-		self.enable_save_as_btn()
+		self.enable_btns()
 	
 	def preset_ctrl_handler(self, event):
 		preset_name = self.preset_ctrl.GetStringSelection()
 		if preset_name:
+			self.set_default_cat()
 			gamma, white, red, green, blue = colormath.rgb_spaces[preset_name]
 			white = colormath.get_whitepoint(white)
 			self._updating_ctrls = True
@@ -492,6 +580,9 @@ class SynthICCFrame(BaseFrame):
 			return "ok"
 		return "invalid"
 
+	def set_default_cat(self):
+		self.cat = "Bradford"
+
 	def set_trc(self, gamma):
 		if gamma == -1023:
 			# DICOM
@@ -525,7 +616,7 @@ class SynthICCFrame(BaseFrame):
 		self.update_trc_controls()
 	
 	def profile_name_ctrl_handler(self, event):
-		self.enable_save_as_btn()
+		self.enable_btns()
 	
 	def red_XYZ_ctrl_handler(self, event):
 		self.parse_XYZ("red")
@@ -630,7 +721,9 @@ class SynthICCFrame(BaseFrame):
 											   (XYZ["wX"], XYZ["wY"], XYZ["wZ"]),
 											   1.0,
 											   "",
-											   getcfg("copyright"))
+											   getcfg("copyright"),
+											   cat=self.cat,
+											   profile_class=profile_class)
 			black = colormath.adapt(XYZ["kX"], XYZ["kY"], XYZ["kZ"], white)
 			profile.tags.rTRC = ICCP.CurveType(profile=profile)
 			profile.tags.gTRC = ICCP.CurveType(profile=profile)
@@ -639,12 +732,17 @@ class SynthICCFrame(BaseFrame):
 		else:
 			# Grayscale profile
 			profile = ICCP.ICCProfile()
+			# Profile class
+			profile.profileClass = profile_class
+			if (not ICCP.s15f16_is_equal((XYZ["wX"], XYZ["wY"], XYZ["wZ"]),
+										 colormath.get_whitepoint("D50")) and
+				(profile.profileClass not in ("mntr", "prtr") or
+				 colormath.is_similar_matrix(colormath.get_cat_matrix(self.cat),
+										     colormath.get_cat_matrix("Bradford")))):
+				profile.version = 2.2  # Match ArgyllCMS
 			profile.colorSpace = "GRAY"
 			profile.setCopyright(getcfg("copyright"))
-			profile.tags.wtpt = ICCP.XYZType()
-			(profile.tags.wtpt.X,
-			 profile.tags.wtpt.Y,
-			 profile.tags.wtpt.Z) = (XYZ["wX"], XYZ["wY"], XYZ["wZ"])
+			profile.set_wtpt((XYZ["wX"], XYZ["wY"], XYZ["wZ"]), self.cat)
 			black = [XYZ["wY"] * (getcfg("synthprofile.black_luminance") /
 								  getcfg("synthprofile.luminance"))] * 3
 			profile.tags.kTRC = ICCP.CurveType(profile=profile)
@@ -698,6 +796,7 @@ class SynthICCFrame(BaseFrame):
 											   for v in black],
 											  getcfg("synthprofile.luminance"),
 											  minmll, maxmll,
+											  getcfg("3dlut.hdr_maxmll_alt_clip"),
 											  rolloff=True,
 											  blend_blackpoint=False)
 				else:
@@ -725,21 +824,24 @@ class SynthICCFrame(BaseFrame):
 									  self.worker.lastmsg])
 					quality = getcfg("profile.quality")
 					clutres = {"m": 17, "l": 9}.get(quality, 33)
-					profile.tags.A2B0 = ICCP.create_synthetic_hdr_clut_profile(
+					hdr_clut_profile = ICCP.create_synthetic_hdr_clut_profile(
 						hdr_format,
 						rgb_space, "",
 						getcfg("synthprofile.black_luminance") * (1 - outoffset),
 						getcfg("synthprofile.luminance"), minmll, maxmll,
+						getcfg("3dlut.hdr_maxmll_alt_clip"),
 						1.2, getcfg("3dlut.hdr_ambient_luminance"),
-						clutres=clutres, worker=self.worker,
-						logfile=logfiles).tags.A2B0
+						clutres=clutres, sat=getcfg("3dlut.hdr_sat"),
+						hue=getcfg("3dlut.hdr_hue"),
+						generate_B2A=trc == -2, worker=self.worker,
+						logfile=logfiles,
+						cat=self.cat)
+					profile.tags.A2B0 = hdr_clut_profile.tags.A2B0
+					if trc == -2:
+						# HLG
+						profile.tags.B2A0 = hdr_clut_profile.tags.B2A0
 				if black != [0, 0, 0] and outoffset and not bpc:
 					profile.apply_black_offset(black)
-				if rolloff or trc == -2:
-					profile.write(path)
-					self.worker.update_profile_B2A(profile, not bpc,
-												   smooth=False,
-												   rgb_space=rgb_space)
 			else:
 				# Grayscale profile
 				if trc == -2084:
@@ -747,12 +849,14 @@ class SynthICCFrame(BaseFrame):
 														(1 - outoffset),
 														getcfg("synthprofile.luminance"),
 														minmll, maxmll,
+														getcfg("3dlut.hdr_maxmll_alt_clip"),
 														rolloff=True)
 				else:
 					# HLG
-					profile.tags.kTRC.set_smpte2084_trc(0,
-														1.2,
-														getcfg("3dlut.hdr_ambient_luminance"))
+					profile.tags.kTRC.set_hlg_trc(0,
+												  getcfg("synthprofile.luminance"),
+												  1.2,
+												  getcfg("3dlut.hdr_ambient_luminance"))
 				if black != [0, 0, 0] and outoffset and not bpc:
 					profile.tags.kTRC.apply_bpc(black[1])
 		elif black != [0, 0, 0]:
@@ -786,8 +890,6 @@ class SynthICCFrame(BaseFrame):
 			(profile.tags[tagname].X,
 			 profile.tags[tagname].Y,
 			 profile.tags[tagname].Z) = X, Y, Z
-		# Profile class
-		profile.profileClass = profile_class
 		# Technology type
 		if tech:
 			profile.tags.tech = ICCP.SignatureType("sig \0\0\0\0" +
@@ -908,6 +1010,12 @@ class SynthICCFrame(BaseFrame):
 		self.black_output_offset_intctrl.SetValue(outoffset)
 		self.lut3d_hdr_minmll_ctrl.SetValue(getcfg("3dlut.hdr_minmll"))
 		self.lut3d_hdr_maxmll_ctrl.SetValue(getcfg("3dlut.hdr_maxmll"))
+		self.lut3d_hdr_maxmll_alt_clip_cb.SetValue(not bool(getcfg("3dlut.hdr_maxmll_alt_clip")))
+		self.lut3d_hdr_sat_ctrl.SetValue(int(round(getcfg("3dlut.hdr_sat") * 100)))
+		self.lut3d_hdr_update_sat_val()
+		hue = int(round(getcfg("3dlut.hdr_sat") * 100))
+		self.lut3d_hdr_hue_ctrl.SetValue(hue)
+		self.lut3d_hdr_hue_intctrl.SetValue(hue)
 		setcfg("3dlut.hdr_peak_luminance", getcfg("synthprofile.luminance"))
 		self.lut3d_hdr_update_diffuse_white()
 		self.lut3d_hdr_ambient_luminance_ctrl.SetValue(getcfg("3dlut.hdr_ambient_luminance"))
@@ -918,9 +1026,14 @@ class SynthICCFrame(BaseFrame):
 		self.lut3d_hdr_maxmll_label.Show(i == 8)  # SMPTE 2084 (PQ)
 		self.lut3d_hdr_maxmll_ctrl.Show(i == 8)  # SMPTE 2084 (PQ)
 		self.lut3d_hdr_maxmll_ctrl_label.Show(i == 8)  # SMPTE 2084 (PQ)
+		self.lut3d_show_hdr_maxmll_alt_clip_ctrl()
 		self.lut3d_hdr_diffuse_white_label.Show(i == 8)  # SMPTE 2084 (PQ)
 		self.lut3d_hdr_diffuse_white_txt.Show(i == 8)  # SMPTE 2084 (PQ)
 		self.lut3d_hdr_diffuse_white_txt_label.Show(i == 8)  # SMPTE 2084 (PQ)
+		sizer = self.lut3d_hdr_sat_ctrl.ContainingSizer
+		sizer.ShowItems(i in (7, 8))  # SMPTE 2084 (PQ)
+		sizer = self.lut3d_hdr_hue_ctrl.ContainingSizer
+		sizer.ShowItems(i in (7, 8))  # SMPTE 2084 (PQ)
 		self.lut3d_hdr_ambient_luminance_label.Show(i == 2)  # HLG
 		self.lut3d_hdr_ambient_luminance_ctrl.Show(i == 2)  # HLG
 		self.lut3d_hdr_ambient_luminance_ctrl_label.Show(i == 2)  # HLG
